@@ -16,5 +16,5 @@ utils::globalVariables(c(".","k","Segment.Level","Segment.Child","Quant.Error",
                         "Current_State","Next_State","is_self_only","nearest_neighbor",
                         "k_numeric","nn_numeric","mae_numeric","k_sort","nn_sort",
                         "Number of Cells","Number of nearest neighbors","mae","status",
-                        "temp_cluster_data","hoverText","lev","nclust","nn",
+                        "temp_cluster_data","hoverText","lev","nclust","nn","formatted_time",
                         "is_overall_best","is_overall_worst","hover_text", "label"))
